# Copyright 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

logger = logging.getLogger('giza.content.extract.models')

from giza.core.inheritance import InheritableContentBase
from giza.content.steps.models import HeadingMixin

class ExtractData(HeadingMixin, InheritableContentBase): 
    @property
    def append(self):
        return self._get_file('append')

    @append.setter
    def append(self, value):
        self._set_file(value, 'append')

    @property
    def prepend(self):
        return self._get_file('prepend')

    @prepend.setter
    def prepend(self, value):
        self._set_file('prepend')

    @property
    def target(self): 
        return os.path.join(self.conf.system.content.extracts.output_dir, self.ref) + '.rst'

    ## The actual implementation for prepend/append
    def _get_file(self, kind):
        if kind in self.state:
            return self.state[kind]
        else: 
            return False

    def _set_file(self, value, kind):
        paths = [ os.path.abspath(value),
                  os.path.join(self.conf.paths.projectroot, value),
                  os.path.join(self.conf.paths.projectroot, 
                               self.conf.paths.branch_source, value),
                  os.path.join(self.conf.paths.projectroot,
                               self.conf.paths.branch_includes, value) ]

        for path in paths:
            if os.path.isfile(path):
                self.state[kind] = path
                return

        logger.error('cannot {0} to non existing file "{1}", skipping.'.format(kind, value))



# Copyright 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import logging

logger = logging.getLogger('giza.content.extract.tasks')

from giza.tools.files import expand_tree, verbose_remove, safe_create_directory
from giza.tools.transformation import append_to_file, prepend_to_file
from giza.content.extract.inheritance import ExtractDataCache
from giza.content.extract.views import render_extracts, get_include_statement
from giza.config.content import new_content_type

def register_extracts(conf):
    conf.system.content.add(name='extracts', definition=new_content_type(name='extract', task_generator=extract_tasks, conf=conf))

def write_extract_file(extract, fn):
    content = render_extract(extract)
    content.write(fn)
    logger.info('wrote extract file: ' + fn)

def extract_tasks(conf, app):
    register_extracts(conf)
    extract_sources = conf.system.content.extracts.sources

    extracts = ExtractDataCache(extract_sources, conf)

    if len(extract_sources) > 0 and not os.path.isdir(conf.system.content.extracts.output_dir):
        safe_create_directory(conf.system.content.extracts.output_dir)

    for dep_fn, extract in extracts.content_iter():
        t = app.add('task')
        t.target = extract.target
        t.dependecy = dep_fn
        t.job = write_extract_file
        t.args = (extract, extract.target)
        t.description = "generating extract file: " + extract.target

        include_statement = get_include_statement(extract.target)

        for verb, adjc, noun in [ (prepend_to_file, 'prepend', extract.prepend),
                                  (append_to_file, 'append', extract.append) ]:
            if noun:
                if not isinstance(noun, list):
                    files = [noun]
                else:
                    files = files

                for fn in files:
                    t = app.add('task')
                    t.target = fn
                    t.dependency = [extract.target, dep_fn]
                    t.job = verb
                    t.args = (fn, include_statement)
                    t.description = "{0} extract include for '{0}' to '{1}'".format(adjc, extract.target, fn)



# 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from giza.config.base import RecursiveConfigurationBase
from giza.tools.serialization import dict_from_list

class ReplacementData(RecursiveConfigurationBase):
    def ingest(self, input_obj): 
        input_obj = self._prep_load_data(input_obj)

        if isinstance(input_obj, dict):
            if 'edition' in input_obj and input_obj['edition'] == self.conf.project.edition:
                super(ReplacementData, self).intest(input_obj)
                return
            else:
                raise TypeError("replacement data is malformed given edition configuration.")
        elif not isinstance(input_obj, list):
            raise TypeError("replacement data must be a list or stream of documents.")
        else:
            mapping = dict_from_list('edition', input_obj)

            if self.conf.project.edition in mapping:
                if isinstance(mapping[self.conf.project.edition], dict):
                    self.state = mapping[self.conf.project.edition]
                else: 
                    TypeError("the replacements for the {0} edition are malformed".format(self.conf.project.edition))
            else: 
                raise TypeError("no replacements configured for edition: " + self.conf.project.edition)

    def items(self):
        return self.state.items()

    def keys(self):
        return self.state.keys()

    def values(self):
        return self.state.values()



# Copyright 2014 MongoDB, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import logging

logger = logging.getLogger('giza.content.release.tasks')

from giza.tools.files import expand_tree, verbose_remove, safe_create_directory
from giza.content.release.inheritance import ReleaseDataCache
from giza.content.release.views import render_releases
from giza.config.content import new_content_type

def register_releases(conf):
    conf.system.content.add(name='releases', definition=new_content_type(name='release', task_generator=release_tasks, conf=conf))

def write_release_file(release, fn, conf):
    content = render_releases(release, conf)
    content.write(fn)
    logger.info('wrote release content: ' + fn)

def release_tasks(conf, app):
    register_releases(conf)
    release_sources = conf.system.content.releases.sources

    rel = ReleaseDataCache(release_sources, conf)

    if len(release_sources) > 0 and not os.path.isdir(conf.system.content.releases.output_dir):
        safe_create_directory(conf.system.content.releases.output_dir)

    for dep_fn, release in rel.content_iter():
        t = app.add('task')
        t.job = write_release_file
        t.target = release.target
        t.dependency = dep_fn
        t.args = (release, release.target, conf)
        t.description = 'generating release spec file: ' + release.target

def release_clean(conf, app):
    register_releases(conf)

    rel = ReleaseDataCache(conf.system.content.releases.sources, conf)

    for dep, release in rel.content_iter():
        task = app.add('task')
        task.target = True
        task.dependency = fn
        task.job = verbose_remove
        task.args = [release.target]
        task.description = 'removing {0}'.format(fn)
